@@ -19,12 +19,11 @@ package EBox::DHCP;
 use strict;
 use warnings;
 
-use base qw(EBox::GConfModule 
-		EBox::NetworkObserver 
-		EBox::LogObserver 
-		EBox::Model::ModelProvider 
-		EBox::Model::CompositeProvider
-		EBox::ServiceModule::ServiceInterface);
+use base qw(EBox::Module::Service
+		EBox::NetworkObserver
+		EBox::LogObserver
+		EBox::Model::ModelProvider
+		EBox::Model::CompositeProvider);
 
 use EBox::Config;
 use EBox::Exceptions::InvalidData;
@@ -35,7 +34,6 @@ use EBox::Global;
 use EBox::Menu::Item;
 use EBox::Menu::Folder;
 use EBox::Objects;
-use EBox::Summary::Status;
 use EBox::Validate qw(:all);
 
 use EBox::Model::ModelManager;
@@ -63,9 +61,11 @@ use Net::IP;
 use HTML::Mason;
 use Error qw(:try);
 use Perl6::Junction qw(any);
+use Text::DHCPLeases;
 
 # Module local conf stuff
 use constant DHCPCONFFILE => "/etc/dhcp3/dhcpd.conf";
+use constant LEASEFILE => "/var/lib/dhcp3/dhcpd.leases";
 use constant PIDFILE => "/var/run/dhcpd.pid";
 use constant DHCP_SERVICE => "ebox.dhcpd3";
 use constant TFTP_SERVICE => "tftpd-hpa";
@@ -82,13 +82,13 @@ use constant TFTPD_CONF_DIR => '/var/lib/tftpboot/';
 #
 # Overrides:
 #
-#    <EBox::GConfModule::_create>
+#    <EBox::Module::Service::_create>
 #
 sub _create
 {
 	my $class = shift;
-	my $self  = $class->SUPER::_create(name => 'dhcp', 
-                                           printableName => 'dhcp server',
+	my $self  = $class->SUPER::_create(name => 'dhcp',
+                                           printableName => 'DHCP',
                                            domain => 'ebox-dhcp',
                                            @_);
 	bless ($self, $class);
@@ -103,7 +103,7 @@ sub domain
 
 # Method: usedFiles
 #
-#	Override EBox::ServiceModule::ServiceInterface::usedFiles
+#	Override EBox::Module::Service::usedFiles
 #
 sub usedFiles
 {
@@ -111,92 +111,42 @@ sub usedFiles
 		{
 		 'file' => DHCPCONFFILE,
 		 'module' => 'dhcp',
- 	 	 'reason' => 'dhcpd configuration file'
+         'reason' => 'dhcpd configuration file'
 		}
 	       ];
 }
 
-#  Method: enableModDepends
+# Method: enableActions
 #
-#   Override EBox::ServiceModule::ServiceInterface::enableModDepends
-#
-sub enableModDepends 
-{
-    return ['network'];
-}
-
-# Method: enableActions 
-#
-# 	Override EBox::ServiceModule::ServiceInterface::enableActions
+# 	Override EBox::Module::Service::enableActions
 #
 sub enableActions
 {
     root(EBox::Config::share() . '/ebox-dhcp/ebox-dhcp-enable');
 }
 
-
-# Method: serviceModuleName 
+# Method: _daemons
 #
-#	Override EBox::ServiceModule::ServiceInterface::serviceModuleName
+#	Override EBox::Module::Service::daemons
 #
-sub serviceModuleName
+sub _daemons
 {
-	return 'dhcp';
+    return [ { 'name' => DHCP_SERVICE } ];
 }
 
-# Method: isRunning
+# Method: _setConf
 #
-#   Override EBox::ServiceModule::ServiceInterface::isRunning
-#
-sub isRunning
-{
-    return EBox::Service::running(DHCP_SERVICE);
-}
-
-# Method: _stopService
-#
-#        Stop the dhcp service
-# Overrides:
-#
-#       <EBox::Module::_stopService>
-#
-sub _stopService
-{
-	EBox::Service::manage(DHCP_SERVICE,'stop');
-}
-
-# Method: _regenConfig
-#
-#      It regenertates the dhcp service configuration
+#      Writes the configuration files
 #
 # Overrides:
 #
-#      <EBox::Module::_regenConfig>
+#      <EBox::Module::Base::_setConf>
 #
-sub _regenConfig
+sub _setConf
 {
     my ($self) = @_;
     $self->_setDHCPConf();
     $self->_setTFTPDConf();
-    $self->_doDaemon();
-}
-
-# Method: statusSummary
-#
-# Overrides:
-#
-#     <EBox::Module::statusSummary>
-#
-# Returns:
-#
-#   <EBox::Summary::Status> - the summary components
-#
-#
-sub statusSummary
-{
-	my $self = shift;
-	return new EBox::Summary::Status('dhcp', 'DHCP',
-		EBox::Service::running(DHCP_SERVICE), $self->service);
 }
 
 # Method: menu
@@ -210,7 +160,9 @@ sub menu
 {
         my ($self, $root) = @_;
         $root->add(new EBox::Menu::Item('url' => 'DHCP/Composite/General',
-                                        'text' => 'DHCP'));
+                                        'text' => $self->printableName(),
+                                        'separator' => __('Infrastructure'),
+                                        'order' => 410));
 }
 
 # Method: models
@@ -328,7 +280,7 @@ sub _exposedMethods
 #
 # Overrides:
 #
-#     <EBox::Model::ModelProvider::composites>
+#     <EBox::Model::CompositeProvider::composites>
 #
 sub composites
 {
@@ -357,62 +309,6 @@ sub composites
 
 }
 
-
-# Method: setService
-#
-#	Set the dhcp service as enabled or disabled
-#
-# Parameters:
-#
-#	enabled - boolean. True enable, undef disable
-#
-sub setService # (enabled)
-{
-    my ($self, $active) = @_;
-
-    $self->enableService($active);
-
-#    ($active and $self->service) and return;
-#    (!$active and !$self->service) and return;
-#
-#    if ($active) {
-#        if ($self->_nStaticIfaces() == 0) {
-#            throw EBox::Exceptions::External(
-#                     __x('DHCP server cannot activated because '
-#                         . 'there are not any network interface '
-#                         . 'with a static address. '
-#                         . '{openhref}Configure one{closehref} first',
-#                         openhref  => "<a href='Network/Ifaces'>",
-#                         closehref => '</a>')
-#                                            );
-#        }
-#    }
-#
-#        #	$self->set_bool("active", $active);
-#        #	$self->_configureFirewall();
-#    $self->{enableModel}->setRow(1,
-#                                 ( enabled => $active)
-#                                );
-}
-
-# Method: service
-#
-#   Return if the dhcp service is enabled
-#
-# Returns:
-#
-#   boolean - true if enabled, otherwise undef
-#
-sub service
-{
-    my ($self) = @_;
-
-    return $self->isEnabled();
-#	return $self->get_bool("active");
-    #return $self->{enableModel}->enabledValue();
-}
-
-
 # Method: initRange
 #
 #	Return the initial host address range for a given interface
@@ -738,14 +634,15 @@ sub ranges # (iface)
 
     my $model = $self->_getModel('rangeModel', $iface);
     my @ranges;
-    for my $row (@{$model->rows()}) {
-        push (@ranges, { 
+    for my $id (@{$model->ids()}) {
+        my $row = $model->row($id);
+        push (@ranges, {
                 name => $row->valueByName('name'),
                 from => $row->valueByName('from'),
                 to => $row->valueByName('to')
                 });
     }
-    
+
     return \@ranges;
 }
 
@@ -774,7 +671,7 @@ sub ranges # (iface)
 #	DataNotFound - Interface does not exist
 #	External - Interface is not configured as static
 #	External - ip is not in the network for the given interface
-#	External - ip overlap 
+#	External - ip overlap
 #	External - ip already configured as fixed
 #
 sub fixedAddressAction
@@ -833,7 +730,7 @@ sub fixedAddressAction
 #
 # Returns:
 #
-#	array ref - contating the fixed addresses in hash refereces. 
+#	array ref - contating the fixed addresses in hash refereces.
 #	Each hash holds the keys 'mac', 'ip' and 'name'
 #
 # Exceptions:
@@ -860,8 +757,9 @@ sub fixedAddresses # (interface)
 
 	my $model = $self->_getModel('fixedAddrModel', $iface);
 	my @addrs;
-	for my $row (@{$model->rows()}) {
-		push (@addrs, { 
+	for my $id (@{$model->ids()}) {
+		my $row = $model->row($id);
+		push (@addrs, {
 				name => $row->valueByName('name'),
 				ip => $row->valueByName('ip'),
 				mac => $row->valueByName('mac')
@@ -1015,13 +913,14 @@ sub vifaceAdded # (iface, viface, address, netmask)
         my @rangeModels = @{$manager->model('/dhcp/RangeTable/*')};
         # Check that the new IP for the virtual interface isn't in any range
         foreach my $rangeModel (@rangeModels) {
-            foreach my $rangeRow (@{$rangeModel->rows()}) {
+            foreach my $id (@{$rangeModel->ids()}) {
+                my $rangeRow = $rangeModel->row($id);
                 my $from = $rangeRow->valueByName('from');
                 my $to   = $rangeRow->valueByName('to');
                 my $range = new Net::IP($from . ' - ' . $to);
                 unless ( $ip->overlaps($range) == $IP_NO_OVERLAP ) {
                     throw EBox::Exceptions::External(
-			__x('The IP address of the virtual interface '
+                    __x('The IP address of the virtual interface '
                             . 'you are trying to add is already used by the '
                             . "DHCP range '{range}' in the interface "
                             . "'{iface}'. Please, remove it before trying "
@@ -1035,7 +934,8 @@ sub vifaceAdded # (iface, viface, address, netmask)
         my @fixedAddrModels = @{$manager->model('/dhcp/FixedAddressTable/*')};
         # Check the new IP for the virtual interface is not a fixed address
         foreach my $fixedAddrModel (@fixedAddrModels) {
-            foreach my $mappingRow (@{$fixedAddrModel->rows()}) {
+            foreach my $id (@{$fixedAddrModel->ids()}) {
+                my $mappingRow = $fixedAddrModel->row($id);
                 my $fixedIP = new Net::IP($mappingRow->valueByName('ip'));
                 unless ( $ip->overlaps($fixedIP) == $IP_NO_OVERLAP ) {
                     throw EBox::Exceptions::External(
@@ -1045,7 +945,7 @@ sub vifaceAdded # (iface, viface, address, netmask)
                                . "interface '{iface}'. Please, remove it "
                                . 'before trying to add a virtual interface '
                                . 'using it.',
-                               fixed => $mappingRow->{plainValueHash}->{name},
+                               fixed => $mappingRow->valueByName('name'),
                                iface => $fixedAddrModel->index()));
 
                 }
@@ -1117,7 +1017,8 @@ sub staticIfaceAddressChanged # (iface, old_addr, old_mask, new_addr, new_mask)
         # Check ranges
         my $manager = EBox::Model::ModelManager->instance();
         my $rangeModel = $manager->model("/dhcp/RangeTable/$iface");
-        foreach my $rangeRow (@{$rangeModel->rows()}) {
+        foreach my $id (@{$rangeModel->ids()}) {
+            my $rangeRow = $rangeModel->row($id);
             my $range = new Net::IP($rangeRow->valueByName('from')
                                     . ' - ' .
                                     $rangeRow->valueByName('to'));
@@ -1131,7 +1032,8 @@ sub staticIfaceAddressChanged # (iface, old_addr, old_mask, new_addr, new_mask)
             }
         }
         my $fixedAddrModel = $manager->model("/dhcp/FixedAddressTable/$iface");
-        foreach my $mappingRow (@{$fixedAddrModel->rows()}) {
+        foreach my $id (@{$fixedAddrModel->ids()}) {
+            my $mappingRow = $fixedAddrModel->row($id);
             my $fixedIP = new Net::IP( $mappingRow->valueByName('ip') );
             # Check the fixed address is still in the network
             unless($fixedIP->overlaps($netIP) == $IP_A_IN_B_OVERLAP){
@@ -1241,8 +1143,8 @@ sub tableInfo {
 	};
 	my @order = ('timestamp', 'ip', 'mac', 'interface', 'event');
 	my $events = {'leased' => __('Leased'), 'released' => __('Released') };
-	
-	return {
+
+	return [{
 		'name' => __('DHCP'),
 		'index' => 'dhcp',
 		'titles' => $titles,
@@ -1252,7 +1154,7 @@ sub tableInfo {
 		'filter' => ['interface', 'mac', 'ip'],
 		'events' => $events,
 		'eventcol' => 'event'
-	};
+	}];
 }
 
 sub logHelper
@@ -1262,30 +1164,83 @@ sub logHelper
 	return (new EBox::DHCPLogHelper);
 }
 
-# Group: Private methods
-
-# Method: _doDaemon
-#
-#      Manage dchp-related services (dhcp3-server and tftpd-hpa)
-#
-# Parameters:
-#
-#      service - String the service to manage. Options: dhcp or tftp
-#
-sub _doDaemon
+sub _leaseIDFromIP
 {
-	my ($self) = @_;
-
-        # dhcp3-server
-	if ($self->service and EBox::Service::running(DHCP_SERVICE)) {
-		EBox::Service::manage(DHCP_SERVICE,'restart');
-	} elsif ($self->service) {
-		EBox::Service::manage(DHCP_SERVICE,'start');
-	} elsif (not $self->service and EBox::Service::running(DHCP_SERVICE)) {
-		EBox::Service::manage(DHCP_SERVICE,'stop');
-	}
+    my ($ip) = @_;
+    my $id = 'a';
+    #force every byte to use 3 digits to make sorting trivial
+    my @bytes = split('\.', $ip);
+    for my $byte (@bytes) {
+        $id .= sprintf("%03d", $byte);
+    }
+    return $id;
 }
 
+sub _dhcpLeases
+{
+    my ($self) = @_;
+    if(!defined($self->{'leases'})) {
+        my $leases = Text::DHCPLeases->new(file => LEASEFILE);
+
+        $self->{'leases'} = {};
+        foreach my $lease ($leases->get_objects()) {
+            my $id = _leaseIDFromIP($lease->ip_address());
+            $self->{'leases'}->{$id} = $lease;
+        }
+    }
+    return $self->{'leases'};
+}
+
+sub _leaseFromIP
+{
+    my ($self, $ip) = @_;
+
+    my $leases = $self->_dhcpLeases();
+    my $id = _leaseIDFromIP($ip);
+    return $leases->{$id};
+}
+
+sub dhcpLeasesWidget
+{
+    my ($self, $widget) = @_;
+    my $section = new EBox::Dashboard::Section('dhcpleases');
+    $widget->add($section);
+    my $titles = [__('IP address'),__('MAC address'), __('Host name')];
+
+    my $leases = $self->_dhcpLeases();
+
+    my $ids = [];
+    my $rows = {};
+    foreach my $id (sort keys (%{$leases})) {
+        my $lease = $leases->{$id};
+        if($lease->binding_state() eq 'active') {
+            my $hostname = $lease->client_hostname();
+            $hostname =~ s/"//g;
+            push(@{$ids}, $id);
+            $rows->{$id} = [$lease->ip_address(),$lease->mac_address(),
+                            $hostname];
+        }
+    }
+    $section->add(new EBox::Dashboard::List(undef, $titles, $ids, $rows));
+}
+
+### Method: widgets
+#
+#   Overrides <EBox::Module::widgets>
+#
+sub widgets
+{
+    return {
+        'dhcpleases' => {
+            'title' => __("DHCP leases"),
+            'widget' => \&dhcpLeasesWidget,
+            'default' => 1
+        }
+    };
+}
+
+# Group: Private methods
+
 # Method: _setDHCPConf
 #
 #     Updates the dhcpd.conf file
@@ -1341,55 +1296,55 @@ sub _ifacesInfo
   my $ifaces = $net->allIfaces();
 
   my %iflist;
-  foreach (@{$ifaces}) {
-    if ($net->ifaceMethod($_) eq 'static') {
-      my $address = $net->ifaceAddress($_);
-      my $netmask = $net->ifaceNetmask($_);
+  foreach my $iface (@{$ifaces}) {
+    if ($net->ifaceMethod($iface) eq 'static') {
+      my $address = $net->ifaceAddress($iface);
+      my $netmask = $net->ifaceNetmask($iface);
       my $network = ip_network($address, $netmask);
 
-      $iflist{$_}->{'net'} = $network;
-      $iflist{$_}->{'address'} = $address;
-      $iflist{$_}->{'netmask'} = $netmask;
-      $iflist{$_}->{'ranges'} = $self->ranges($_);
-      $iflist{$_}->{'fixed'} = $self->fixedAddresses($_);
+      $iflist{$iface}->{'net'} = $network;
+      $iflist{$iface}->{'address'} = $address;
+      $iflist{$iface}->{'netmask'} = $netmask;
+      $iflist{$iface}->{'ranges'} = $self->ranges($iface);
+      $iflist{$iface}->{'fixed'} = $self->fixedAddresses($iface);
 
       # look if we have static routes for this network
       my $netWithMask = EBox::NetWrappers::to_network_with_mask($network, $netmask);
-       $iflist{$_}->{'staticRoutes'} =  $staticRoutes_r->{$netWithMask} if exists $staticRoutes_r->{$netWithMask};
+       $iflist{$iface}->{'staticRoutes'} =  $staticRoutes_r->{$netWithMask} if exists $staticRoutes_r->{$netWithMask};
 
-      my $gateway = $self->defaultGateway($_);
+      my $gateway = $self->defaultGateway($iface);
       if (defined($gateway) and $gateway ne "") {
-	$iflist{$_}->{'gateway'} = $gateway;
+	$iflist{$iface}->{'gateway'} = $gateway;
       } else {
-	$iflist{$_}->{'gateway'} = $address;
+	$iflist{$iface}->{'gateway'} = $address;
       }
-      my $search = $self->searchDomain($_);
-      $iflist{$_}->{'search'} = $search;
-      my $nameserver1 = $self->nameserver($_,1);
+      my $search = $self->searchDomain($iface);
+      $iflist{$iface}->{'search'} = $search;
+      my $nameserver1 = $self->nameserver($iface,1);
       if (defined($nameserver1) and $nameserver1 ne "") {
-	$iflist{$_}->{'nameserver1'} = $nameserver1;
+	$iflist{$iface}->{'nameserver1'} = $nameserver1;
       }
-      my $nameserver2 = $self->nameserver($_,2);
+      my $nameserver2 = $self->nameserver($iface,2);
       if (defined($nameserver2) and $nameserver2 ne "") {
-	$iflist{$_}->{'nameserver2'} = $nameserver2;
+	$iflist{$iface}->{'nameserver2'} = $nameserver2;
       }
       # Leased times
-      my $defaultLeasedTime = $self->_leasedTime('default', $_);
+      my $defaultLeasedTime = $self->_leasedTime('default', $iface);
       if (defined($defaultLeasedTime)) {
-        $iflist{$_}->{'defaultLeasedTime'} = $defaultLeasedTime;
+        $iflist{$iface}->{'defaultLeasedTime'} = $defaultLeasedTime;
       }
-      my $maxLeasedTime = $self->_leasedTime('max', $_);
+      my $maxLeasedTime = $self->_leasedTime('max', $iface);
       if (defined($maxLeasedTime)) {
-        $iflist{$_}->{'maxLeasedTime'} = $maxLeasedTime;
+        $iflist{$iface}->{'maxLeasedTime'} = $maxLeasedTime;
       }
       # Thin client options
-      my $nextServer = $self->_thinClientOption('nextServer', $_);
+      my $nextServer = $self->_thinClientOption('nextServer', $iface);
       if ($nextServer) {
-        $iflist{$_}->{'nextServer'} = $nextServer;
+        $iflist{$iface}->{'nextServer'} = $nextServer;
       }
-      my $filename = $self->_thinClientOption('filename', $_);
+      my $filename = $self->_thinClientOption('filename', $iface);
       if (defined($filename)) {
-        $iflist{$_}->{'filename'} = $filename;
+        $iflist{$iface}->{'filename'} = $filename;
       }
     }
   }
@@ -1414,9 +1369,9 @@ sub _realIfaces
 
   my $real_ifaces = $net->ifaces();
   my %realifs;
-  foreach (@{$real_ifaces}) {
-    if ($net->ifaceMethod($_) eq 'static') {
-      $realifs{$_} = $net->vifaceNames($_);
+  foreach my $iface (@{$real_ifaces}) {
+    if ($net->ifaceMethod($iface) eq 'static') {
+      $realifs{$iface} = $net->vifaceNames($iface);
     }
 
   }
@@ -1443,7 +1398,6 @@ sub _realIfaces
 #
 sub _areThereThinClientOptions
 {
-
     my ($self, $ifacesInfo) = @_;
 
     foreach my $iface (keys %{$ifacesInfo}) {
@@ -1454,7 +1408,6 @@ sub _areThereThinClientOptions
         }
     }
     return 0;
-
 }
 
 # Method: _leasedTime
@@ -1463,14 +1416,12 @@ sub _areThereThinClientOptions
 #
 sub _leasedTime # (which, iface)
 {
-
     my ($self, $which, $iface) = @_;
 
     my $advOptionsModel = $self->_getModel('leaseTimesModel', $iface);
 
     my $fieldName = $which . '_leased_time';
-    return $advOptionsModel->row()->{plainValueHash}->{$fieldName};
-
+    return $advOptionsModel->row()->valueByName($fieldName);
 }
 
 # Method: _thinClientOption
@@ -1485,12 +1436,17 @@ sub _thinClientOption # (option, iface)
     my $thinClientModel = $self->_getModel('thinClientModel', $iface);
 
     if ( $option eq 'filename' ) {
-        my $fileType = $thinClientModel->row()->{valueHash}->{filename};
+        my $fileType = $thinClientModel->row()->elementByName('filename');
         if ( $fileType->exist() ) {
             # Write down the one stored in tftpd
             return EBox::DHCP->TftpdRootDir() . $iface . '_firmware';
         } else {
-            return undef;
+            my $file = $thinClientModel->row()->valueByName('remoteFilename');
+            if( defined($file) and ($file ne '') ) {
+                return $file;
+            } else {
+                return undef;
+            }
         }
     } else {
         return $thinClientModel->nextServer();
@@ -1510,7 +1466,7 @@ sub _configureFirewall {
 		$fw->removeOutputRule('tcp', 68);
 	} catch EBox::Exceptions::Internal with { };
 
-	if ($self->service) {
+	if ($self->isEnabled()) {
 		$fw->addOutputRule('tcp', 67);
 		$fw->addOutputRule('tcp', 68);
 		$fw->addOutputRule('udp', 67);
@@ -1556,8 +1512,8 @@ sub _checkStaticIfaces
 
   my $nStaticIfaces = $self->_nStaticIfaces() + $adjustNumber;
   if ($nStaticIfaces == 0) {
-    if ($self->service()) {
-      $self->setService(0);
+    if ($self->isEnabled()) {
+      $self->enableService(0);
       EBox::info('DHCP service was deactivated because there was not any static interface left');
     }
   }
@@ -1575,7 +1531,6 @@ sub _nStaticIfaces
   return $staticIfaces;
 }
 
-
 # Method:  userConfDir
 #
 #  Returns:
@@ -1585,5 +1540,4 @@ sub userConfDir
   return CONF_DIR;
 }
 
-
- 1;
+1;
