@@ -165,9 +165,8 @@ sub validateTypedRow
         # Check if chosen is DNS to check if it's enabled
         if ( $changedFields->{primary_ns}->selectedType() eq 'eboxDNS' ) {
             my $dns = EBox::Global->modInstance('dns');
-            unless ( $dns->service() ) {
-                throw EBox::Exceptions::External(__('DNS service must be active to as primary '
-                                                    . 'nameserver the local eBox DNS server'));
+            unless ( $dns->isEnabled() ) {
+                throw EBox::Exceptions::External(__('DNS service must be active to be able to select eBox as primary DNS server'));
             }
         }
     }
@@ -378,19 +377,26 @@ sub _table
                                                        foreignModel  => \&_gatewayModel,
                                                        foreignField  => 'name'
                                                       ),
-                              ]
+                              ],
+                              help          => __('Setting "eBox" as default gateway will set the interface '
+                                                  . 'IP address as gateway'),
+
                              ),
        new EBox::Types::Union(
                               fieldName     => 'search_domain',
                               printableName => __('Search domain'),
                               editable      => 1,
                               subtypes      => \@searchDomainSubtypes,
+                              help          => __('The selected domain will complete on your clients '
+                                                  . 'those DNS queries which are not fully qualified'),
                              ),
        new EBox::Types::Union(
                               fieldName      => 'primary_ns',
                               printableName  => __('Primary nameserver'),
                               editable       => 1,
                               subtypes       => \@primaryNSSubtypes,
+                              help           => __('If "eBox DNS" is present and selected, the eBox server will act '
+                                                   . 'as cache DNS server'),
                              ),
        new EBox::Types::HostIP(
                                fieldName     => 'secondary_ns',
@@ -407,14 +413,11 @@ sub _table
                       defaultActions     => [ 'editField', 'changeView' ],
                       tableDescription   => \@tableDesc,
                       class              => 'dataForm',
-                      help               => __('Setting "eBox" as default gateway will set '
-                                               . 'as default gateway the interface address. '
-                                               . 'If you set a "name", you may choose one the configured '
-                                               . 'gateways. As "search domain" value, '
-                                               . 'one of the configured DNS domains on eBox might be chosen. '
-                                               . 'If you set the "Primary nameserver" the "eBox '
-                                               . 'DNS" if installed, the eBox server may act as '
-                                               . 'cache DNS server. All fields are optionals setting '
+                      help               => __('If you set a "configured ones" as default gateway, '
+                                               . 'you may choose one the configured gateways. As '
+                                               . '"search domain" value, one of the configured '
+                                               . 'DNS domains on eBox might be chosen. '
+                                               . 'All fields are optionals setting '
                                                . 'its value as "None" or leaving blank.'),
                      };
 
